"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var src_1 = require("../../src");
var execution_result_processor_1 = require("../../src/services/execution-result-processor");
var null_logger_1 = require("../../src/services/null-logger");
describe("ExecutionResultProcessor", function () {
    var subject = new execution_result_processor_1.ExecutionResultProcessor();
    subject.logger = new null_logger_1.NullLogger();
    beforeEach(function () {
        subject.pointerFactory = { buildNextPointer: function () { } };
    });
    it("should advance workflow", function () {
        //arrange
        var pointer1 = new src_1.ExecutionPointer();
        pointer1.active = true;
        pointer1.stepId = 0;
        pointer1.status = src_1.PointerStatus.Running;
        var pointer2 = new src_1.ExecutionPointer();
        var outcome = new src_1.StepOutcome();
        outcome.nextStep = 1;
        var step = new src_1.WorkflowStep();
        step.outcomes.push(outcome);
        var instance = givenWorkflow([pointer1]);
        var wfResult = new src_1.WorkflowExecutorResult();
        var stepResult = new src_1.ExecutionResult();
        stepResult.proceed = true;
        stepResult.outcomeValue = null;
        spyOn(subject.pointerFactory, "buildNextPointer").and.returnValue(pointer2);
        //act
        subject.processExecutionResult(stepResult, pointer1, instance, step, wfResult);
        //assert
        expect(pointer1.active).toBe(false);
        expect(pointer1.status).toBe(src_1.PointerStatus.Complete);
        expect(pointer1.endTime).toBeDefined();
        expect(instance.executionPointers).toContain(pointer2);
        expect(subject.pointerFactory.buildNextPointer).toHaveBeenCalled();
    });
    it("should set persistence data", function () {
        //arrange
        var data = new Object();
        var pointer = new src_1.ExecutionPointer();
        pointer.active = true;
        pointer.stepId = 0;
        pointer.status = src_1.PointerStatus.Running;
        var step = new src_1.WorkflowStep();
        var instance = givenWorkflow([pointer]);
        var wfResult = new src_1.WorkflowExecutorResult();
        var stepResult = new src_1.ExecutionResult();
        stepResult.proceed = false;
        stepResult.persistenceData = data;
        //act
        subject.processExecutionResult(stepResult, pointer, instance, step, wfResult);
        //assert
        expect(pointer.persistenceData).toBe(data);
    });
    it("should subscribe to event", function () {
        //arrange
        var pointer = new src_1.ExecutionPointer();
        pointer.active = true;
        pointer.stepId = 0;
        pointer.status = src_1.PointerStatus.Running;
        var step = new src_1.WorkflowStep();
        var instance = givenWorkflow([pointer]);
        var wfResult = new src_1.WorkflowExecutorResult();
        var stepResult = new src_1.ExecutionResult();
        stepResult.proceed = false;
        stepResult.eventKey = "key";
        stepResult.eventName = "event";
        //act
        subject.processExecutionResult(stepResult, pointer, instance, step, wfResult);
        //assert
        expect(pointer.active).toBe(false);
        expect(pointer.status).toBe(src_1.PointerStatus.WaitingForEvent);
        expect(pointer.eventName).toBe(stepResult.eventName);
        expect(pointer.eventKey).toBe(stepResult.eventKey);
        expect(wfResult.subscriptions.length).toBe(1);
    });
});
function givenWorkflow(pointers) {
    var result = new src_1.WorkflowInstance();
    result.status = src_1.WorkflowStatus.Runnable;
    result.executionPointers.push(pointers);
    return result;
}
//# sourceMappingURL=execution-result-processor.spec.js.map