"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var models_1 = require("../models");
var step_builder_1 = require("./step-builder");
var WorkflowBuilder = /** @class */ (function () {
    function WorkflowBuilder() {
        this.steps = [];
        this.errorBehavior = models_1.WorkflowErrorHandling.Retry;
        this.retryInterval = (60 * 1000);
    }
    WorkflowBuilder.prototype.build = function (id, version) {
        var result = new models_1.WorkflowDefinition();
        result.id = id;
        result.version = version;
        result.steps = this.steps;
        result.errorBehavior = this.errorBehavior;
        result.retryInterval = this.retryInterval;
        return result;
    };
    WorkflowBuilder.prototype.addStep = function (step) {
        step.id = this.steps.length;
        this.steps.push(step);
    };
    WorkflowBuilder.prototype.startWith = function (body, setup) {
        if (setup === void 0) { setup = null; }
        var step = new models_1.WorkflowStep();
        step.body = body;
        var stepBuilder = new step_builder_1.StepBuilder(this, step);
        //setup
        if (setup) {
            setup(stepBuilder);
        }
        this.addStep(step);
        return stepBuilder;
    };
    WorkflowBuilder.prototype.getUpstreamSteps = function (id) {
        return this.steps.filter(function (step) { return step.outcomes.filter(function (outcome) { return outcome.nextStep == id; }).length > 0; });
    };
    WorkflowBuilder.prototype.lastStep = function () {
        var last = this.steps.reduce(function (prev, current) { return prev.id > current.id ? prev : current; });
        return last.id;
    };
    return WorkflowBuilder;
}());
exports.WorkflowBuilder = WorkflowBuilder;
//# sourceMappingURL=workflow-builder.js.map