"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
var inversify_1 = require("inversify");
var abstractions_1 = require("../abstractions");
var models_1 = require("../models");
var util_1 = require("util");
var ExecutionResultProcessor = /** @class */ (function () {
    function ExecutionResultProcessor() {
    }
    ExecutionResultProcessor.prototype.processExecutionResult = function (stepResult, pointer, instance, step, workflowResult) {
        pointer.persistenceData = stepResult.persistenceData;
        pointer.outcome = stepResult.outcomeValue;
        if (stepResult.sleep) {
            pointer.sleepUntil = stepResult.sleep.getTime();
            pointer.status = models_1.PointerStatus.Sleeping;
        }
        if (stepResult.eventName) {
            pointer.eventName = stepResult.eventName;
            pointer.eventKey = stepResult.eventKey;
            pointer.active = false;
            pointer.status = models_1.PointerStatus.WaitingForEvent;
            var subscription = new models_1.EventSubscription();
            subscription.workflowId = instance.id;
            subscription.stepId = pointer.stepId;
            subscription.eventName = pointer.eventName;
            subscription.eventKey = pointer.eventKey;
            subscription.subscribeAsOf = stepResult.eventAsOf;
            workflowResult.subscriptions.push(subscription);
        }
        if (stepResult.proceed) {
            pointer.active = false;
            pointer.status = models_1.PointerStatus.Complete;
            pointer.endTime = new Date();
            for (var _i = 0, _a = step.outcomes.filter(function (x) { return (x.value(instance.data) == stepResult.outcomeValue) || (x.value(instance.data) == null); }); _i < _a.length; _i++) {
                var outcome = _a[_i];
                var newPointer = this.pointerFactory.buildNextPointer(pointer, outcome);
                instance.executionPointers.push(newPointer);
            }
        }
        else {
            for (var _b = 0, _c = stepResult.branchValues; _b < _c.length; _b++) {
                var branch = _c[_b];
                for (var _d = 0, _e = step.children; _d < _e.length; _d++) {
                    var childDefId = _e[_d];
                    var childPointer = this.pointerFactory.buildChildPointer(pointer, childDefId, branch);
                    instance.executionPointers.push(childPointer);
                }
            }
        }
    };
    ExecutionResultProcessor.prototype.handleStepException = function (workflow, definition, pointer, step) {
        pointer.status = models_1.PointerStatus.Failed;
        var errorOption = step.errorBehavior;
        if (!errorOption) {
            if (this.shouldCompensate(workflow, definition, pointer))
                errorOption = models_1.WorkflowErrorHandling.Compensate;
            else
                errorOption = definition.errorBehavior;
        }
        this.selectErrorStrategy(errorOption, workflow, definition, pointer, step);
    };
    ExecutionResultProcessor.prototype.selectErrorStrategy = function (errorOption, workflow, definition, pointer, step) {
        switch (errorOption) {
            case models_1.WorkflowErrorHandling.Retry:
                pointer.sleepUntil = (Date.now() + step.retryInterval);
                step.primeForRetry(pointer);
                break;
            case models_1.WorkflowErrorHandling.Suspend:
                workflow.status = models_1.WorkflowStatus.Suspended;
                break;
            case models_1.WorkflowErrorHandling.Terminate:
                workflow.status = models_1.WorkflowStatus.Terminated;
                break;
            case models_1.WorkflowErrorHandling.Compensate:
                this.compensate(workflow, definition, pointer);
                break;
            default:
                pointer.sleepUntil = (Date.now() + 60000);
                break;
        }
        pointer.retryCount++;
    };
    ExecutionResultProcessor.prototype.compensate = function (workflow, definition, exceptionPointer) {
        var scope = [];
        if (exceptionPointer.scope)
            scope = exceptionPointer.scope.slice();
        scope.push(exceptionPointer.id);
        var _loop_1 = function () {
            var pointerId = scope.pop();
            var pointer = workflow.executionPointers.find(function (x) { return x.id == pointerId; });
            var step = definition.steps.find(function (x) { return x.id == pointer.stepId; });
            var resume = true;
            var revert = false;
            var txnStack = scope.slice();
            var _loop_2 = function () {
                var parentId = txnStack.pop();
                var parentPointer = workflow.executionPointers.find(function (x) { return x.id == parentId; });
                var parentStep = definition.steps.find(function (x) { return x.id == parentPointer.stepId; });
                if ((!parentStep.resumeChildrenAfterCompensation()) || (parentStep.revertChildrenAfterCompensation())) {
                    resume = parentStep.resumeChildrenAfterCompensation();
                    revert = parentStep.revertChildrenAfterCompensation();
                }
            };
            while (txnStack.length > 0) {
                _loop_2();
            }
            var errorBehavior = this_1.isNull(step.errorBehavior, models_1.WorkflowErrorHandling.Compensate);
            if (errorBehavior != models_1.WorkflowErrorHandling.Compensate) {
                this_1.selectErrorStrategy(this_1.isNull(step.errorBehavior, models_1.WorkflowErrorHandling.Retry), workflow, definition, pointer, step);
                return "continue";
            }
            exceptionPointer.active = false;
            exceptionPointer.endTime = new Date();
            exceptionPointer.status = models_1.PointerStatus.Failed;
            if (!util_1.isNullOrUndefined(step.compensationStepId)) {
                pointer.status = models_1.PointerStatus.Compensated;
                var compensationPointer_1 = this_1.pointerFactory.buildCompensationPointer(pointer, exceptionPointer, step.compensationStepId);
                workflow.executionPointers.push(compensationPointer_1);
                if (resume) {
                    for (var _i = 0, _a = step.outcomes.filter(function (x) { return util_1.isNullOrUndefined(x.value(workflow.data)); }); _i < _a.length; _i++) {
                        var outcomeTarget = _a[_i];
                        workflow.executionPointers.push(this_1.pointerFactory.buildNextPointer(pointer, outcomeTarget));
                    }
                }
            }
            if (revert) {
                var prevSiblings = workflow.executionPointers.filter(function (x) { return JSON.stringify(pointer.scope) == JSON.stringify(x.scope) && x.id != pointer.id && x.status == models_1.PointerStatus.Complete; });
                var _loop_3 = function (siblingPointer) {
                    var siblingStep = definition.steps.find(function (x) { return x.id == siblingPointer.stepId; });
                    if (!util_1.isNullOrUndefined(siblingStep.compensationStepId)) {
                        compensationPointer = this_1.pointerFactory.buildCompensationPointer(siblingPointer, exceptionPointer, siblingStep.compensationStepId);
                        workflow.executionPointers.push(compensationPointer);
                        siblingPointer.status = models_1.PointerStatus.Compensated;
                    }
                };
                for (var _b = 0, prevSiblings_1 = prevSiblings; _b < prevSiblings_1.length; _b++) {
                    var siblingPointer = prevSiblings_1[_b];
                    _loop_3(siblingPointer);
                }
            }
        };
        var this_1 = this, compensationPointer;
        while (scope.length > 0) {
            _loop_1();
        }
    };
    ExecutionResultProcessor.prototype.shouldCompensate = function (workflow, definition, currentPointer) {
        var scope = [];
        if (currentPointer.scope)
            scope = currentPointer.scope.slice();
        scope.push(currentPointer.id);
        var _loop_4 = function () {
            var pointerId = scope.pop();
            var pointer = workflow.executionPointers.find(function (x) { return x.id == pointerId; });
            var step = definition.steps.find(function (x) { return x.id == pointer.stepId; });
            if (step.revertChildrenAfterCompensation)
                return { value: true };
            if ((step.compensationStepId !== undefined) && (step.compensationStepId !== null))
                return { value: true };
        };
        while (scope.length > 0) {
            var state_1 = _loop_4();
            if (typeof state_1 === "object")
                return state_1.value;
        }
        return false;
    };
    ExecutionResultProcessor.prototype.isNull = function (obj, fallback) {
        if (util_1.isNullOrUndefined(obj))
            return fallback;
        return obj;
    };
    __decorate([
        inversify_1.inject(abstractions_1.TYPES.IExecutionPointerFactory),
        __metadata("design:type", Object)
    ], ExecutionResultProcessor.prototype, "pointerFactory", void 0);
    __decorate([
        inversify_1.inject(abstractions_1.TYPES.ILogger),
        __metadata("design:type", Object)
    ], ExecutionResultProcessor.prototype, "logger", void 0);
    ExecutionResultProcessor = __decorate([
        inversify_1.injectable()
    ], ExecutionResultProcessor);
    return ExecutionResultProcessor;
}());
exports.ExecutionResultProcessor = ExecutionResultProcessor;
//# sourceMappingURL=execution-result-processor.js.map